@@ -206,7 +206,7 @@ def sampledata_sql(kwargs: dict) -> None:
 
     # for sample data get project data id
     query = "SELECT id FROM project_data WHERE request_id=%s"
-    db_connect.cur.execute(query, (kwargs["Project Intake service ID"],))
+    db_connect.cur.execute(query, (kwargs["proj_request_id"],))
     projectData_id = [i[0] for i in db_connect.cur.fetchall()]
     if not projectData_id:
         sys.exit("ERROR: missing project id associated with the sample/s !!")
@@ -236,8 +236,8 @@ def sampledata_sql(kwargs: dict) -> None:
             ), "Species does not match registry in DB: {}".format(
                 kwargs["samples"][sample]["Species"]
             )
-            assert "Sample type" in kwargs, "'Sample type' not found {}".format(
-                "kwargs"
+            assert "Sample type" in kwargs, "'Sample type' not found in {}".format(
+                kwargs
             )
         except AssertionError as e:
             sys.exit("AssertionError: {}".format(str(e)))
@@ -341,30 +341,30 @@ def sampledata_sql(kwargs: dict) -> None:
         statements.append([stmt.appropriate_case, kargs])
 
         # make insert statement for hashtag barcodes
-        if "hastag_parameters" in kwargs["samples"][sample]:
-            kwargs["samples"][sample]["hastag_parameters"]["hashtagBarcodes_id"] = {}
-            for k in kwargs["samples"][sample]["hastag_parameters"]["hashtags"]:
-                seq = kwargs["samples"][sample]["hastag_parameters"]["hashtags"][k]
-                bar = kwargs["samples"][sample]["hastag_parameters"]["barcodes"][k]
+        if "hashtag_parameters" in kwargs["samples"][sample]:
+            kwargs["samples"][sample]["hashtag_parameters"]["hashtagBarcodes_id"] = {}
+            for k in kwargs["samples"][sample]["hashtag_parameters"]["hashtags"]:
+                seq = kwargs["samples"][sample]["hashtag_parameters"]["hashtags"][k]
+                bar = kwargs["samples"][sample]["hashtag_parameters"]["barcodes"][k]
                 hashtagBarcodes_id = get_hashtagBarcodes_id(seq, bar)
-                kwargs["samples"][sample]["hastag_parameters"][
+                kwargs["samples"][sample]["hashtag_parameters"][
                     "hashtagBarcodes_id"
                 ].update({k: hashtagBarcodes_id})
             # new cell hashing table
             _columns = ["hashtagBarcodes_id", "demultiplex_label", "notes"]
             _labels = ["hashtagBarcodes_id", "hlabels", "hashtag_notes"]
-            for k in kwargs["samples"][sample]["hastag_parameters"][
+            for k in kwargs["samples"][sample]["hashtag_parameters"][
                 "hashtagBarcodes_id"
             ].keys():
                 kargs = {"sampleData_id": kwargs["samples"][sample]["sampleData_id"]}
                 for label, column in zip(_labels, _columns):
                     try:
                         if (
-                            kwargs["samples"][sample]["hastag_parameters"][label][k]
+                            kwargs["samples"][sample]["hashtag_parameters"][label][k]
                             != "NULL"
                         ):
                             kargs[column] = kwargs["samples"][sample][
-                                "hastag_parameters"
+                                "hashtag_parameters"
                             ][label][k]
                     except KeyError:
                         pass
@@ -559,7 +559,7 @@ def statsdata_sql(kwargs: dict) -> None:
             hto_Tech = hto_Tech[0] if hto_Tech else None
             tcr_Tech = [i for i in sc_mode if i == "VDJ"]
             tcr_Tech = tcr_Tech[0] if tcr_Tech else None
-        if kwargs["Sequencing nuclei (Nuc-seq)?"].lower() == "yes":
+        if kwargs["nucseq"].lower() == "yes":
             sc_Tech = sc_Tech + "_snRNA-seq"
         # make sure all samples are able to pass the following statement
         try:
@@ -677,14 +677,11 @@ def statsdata_sql(kwargs: dict) -> None:
             print(kargs)
             print("Missing `QC` and `sampleIndex_id`")
 
-        if (
-            "Hashtag Parameters" in kwargs
-            and kwargs["Hashtag Parameters"].lower() == "yes"
-        ):
-            kwargs["samples"][sample]["hastag_parameters"][
+        if "hashtag_parameters" in kwargs["samples"][sample]:
+            kwargs["samples"][sample]["hashtag_parameters"][
                 "Hash_sampleIndex_id"
             ] = get_sampleIndex(
-                kwargs["samples"][sample]["hastag_parameters"]["Hashtag Index"]
+                kwargs["samples"][sample]["hashtag_parameters"]["Hashtag Index"]
             )
 
             _columns = ["Sample", "sampleIndex_id", "hash_lib_conct", "notes"]
@@ -697,8 +694,8 @@ def statsdata_sql(kwargs: dict) -> None:
             kargs_ = {"sampleData_id": _kargs[sample]}
             for label, column in zip(_labels, _columns):
                 try:
-                    if kwargs["samples"][sample]["hastag_parameters"][label] != "NULL":
-                        kargs_[column] = kwargs["samples"][sample]["hastag_parameters"][
+                    if kwargs["samples"][sample]["hashtag_parameters"][label] != "NULL":
+                        kargs_[column] = kwargs["samples"][sample]["hashtag_parameters"][
                             label
                         ]
                 except KeyError:
@@ -709,7 +706,19 @@ def statsdata_sql(kwargs: dict) -> None:
             # make sure all samples are able to pass the following statement
             try:
                 # in cases other than 5-prime set hto_Tech = sc_Tech + '_HashTag'
-                hto_Tech = hto_Tech if hto_Tech else sc_Tech + "_HashTag"
+                if hto_Tech is None:
+                    if (
+                        kwargs["samples"][sample]["hashtag_parameters"]["Appended Name"]
+                        == "HTO"
+                    ):
+                        hto_Tech = sc_Tech + "_HashTag"
+                    elif (
+                        kwargs["samples"][sample]["hashtag_parameters"]["Appended Name"]
+                        == "CITE"
+                    ):
+                        hto_Tech = sc_Tech + "_CiteSeq"
+                    else:
+                        hto_Tech = "multiome"
                 stmt = (
                     "UPDATE hashtag_lib SET genomeIndex_id = ("
                     "SELECT genome_index.id FROM genome_index "
@@ -732,7 +741,7 @@ def statsdata_sql(kwargs: dict) -> None:
                     "SELECT genome_index.id ....... FAILED!\n\t{} ".format(str(e))
                 )
 
-        if "TCR Parameters" in kwargs and kwargs["TCR Parameters"].lower() == "yes":
+        if "TCR_parameters" in kwargs["samples"][sample]:
             kwargs["samples"][sample]["TCR_parameters"][
                 "TCR_sampleIndex_id"
             ] = get_sampleIndex(
