@@ -3,18 +3,20 @@
 import os
 import sys
 import re
+import datetime
 import fnmatch
 import shlex
 import hashlib
 import urllib.parse
 import subprocess
 import logging
-from typing import Union, Tuple, Iterable, Optional, Dict
+from typing import Union, Tuple, Iterable, Optional, Dict, List
 from collections import namedtuple, ChainMap
 
 import json
 import yaml
 import boto3
+import mysql
 import numpy as np
 import pandas as pd
 
@@ -503,6 +505,39 @@ def sample_data_frame(sd: pd.DataFrame) -> pd.DataFrame:
 
     table_dict = {"HTO": "hashtag_lib", "TCR": "TCR_lib", None: "sample_data"}
     sample_data["label"] = sample_data["label"].map(table_dict)
+
+    sample_data["source"] = (
+        "/"
+        + sample_data["project_loc"].str.cat(sample_data["old_basename"], sep="/")
+        + "/"
+    )
+    sample_data["s3uri"] = (
+        sample_data["s3_loc"].str.cat(sample_data["sample"], sep="/") + "/"
+    )
+    sample_data["igo_id"] = sample_data.project_loc.str.lstrip(
+        "/Volumes/peerd/FASTQ/Project_"
+    )
+    sample_data["igo_id"] = sample_data.igo_id.str.split(pat="/", n=1, expand=True)[0]
+
+    di = {
+        "ten_x_v2": 1,
+        "ten_x_v3": 1,
+        "H3": 2,
+        "H2": 2,
+        "H4": 2,
+        "CiteSeq": 3,
+        "VDJ": 4,
+        "five_prime": 5,
+        "ATAC": 6,
+        "CR": 7,
+        "in_drop_v2": 8,
+        "in_drop_v4": 8,
+        "in_drop_v5": 8,
+        "multiome": 9,
+    }
+    sample_data["platform"] = "NULL"
+    sample_data["platform"] = sample_data["run_name"].map(di)
+
     return sample_data
 
 
@@ -547,6 +582,7 @@ def prepare_statements(sample_data: pd.DataFrame) -> dict:
                 "H3",
                 "H2",
                 "H4",
+                "CiteSeq",
                 "VDJ",
                 "five_prime",
                 "CR",
@@ -634,7 +670,10 @@ def put_object(
     if md5sums:
         if isinstance(md5sums, str) and os.path.isfile(md5sums):
             with open(md5sums, "r") as f:
-                md5_hashes = {n[1]: n[0] for n in [i.split() for i in f.readlines()]}
+                md5_hashes = {
+                    os.path.basename(n[1]): n[0]
+                    for n in [i.split() for i in f.readlines()]
+                }
         else:
             try:
                 assert isinstance(md5sums, dict), (
@@ -646,7 +685,7 @@ def put_object(
             except AssertionError as e:
                 print(str(e))
                 return b"Checksum failed", False
-        if obj.e_tag == md5_hashes[os.path.basename(src_data)]:
+        if obj.e_tag.strip('"') == md5_hashes[os.path.basename(src_data)].strip('"'):
             logging.info(
                 "--- Added %s to %s",
                 os.path.basename(dest_object_name),
@@ -657,9 +696,9 @@ def put_object(
             logging.error(
                 "Checksum failed %s (%s) to %s (%s)!!",
                 src_data,
-                md5_hashes[os.path.basename(src_data)],
+                md5_hashes[os.path.basename(src_data)].strip('"'),
                 dest_object_name,
-                obj.e_tag,
+                obj.e_tag.strip('"'),
             )
             return b"Checksum failed", False
     else:
@@ -682,6 +721,89 @@ def put_object(
             return b"", True
 
 
+def copy_files_to_processing_folder(sample_data: pd.DataFrame, **kwargs) -> List:
+    """\
+    Method to copy IGO sequencing data from the mounted drive to S3.
+
+    :param sample_data:
+        Data frame of samples to be processed
+    :param kwargs:
+        Additional args passed to other methods.
+
+    :return:
+        List of failed samples
+    """
+    failed_samples = []
+    # copy fastq files, rename the samples, and update the database
+    sample_set = prepare_statements(sample_data)
+    for key in sample_set:
+
+        # before any copying to S3 assert IGO dates are are not missing
+        db_connect.cur.execute(
+            "SELECT IGO_sub_date, sequencing_date FROM important_dates "
+            "WHERE sampleData_id = %s",
+            (key.split("_")[0],),
+        )
+        res = db_connect.cur.fetchall()
+        if not res:
+            logging.warning(
+                "No records found for sample {}:\n\t{}".format(
+                    key.split("_")[0], db_connect.cur.statement
+                )
+            )
+            logging.warning(
+                "Sample '{}' will be excluded from processing!".format(
+                    key.split("_")[0]
+                )
+            )
+            failed_samples.append(str(key.split("_")[0]))
+            logging.warning("Fix the missing dates and try again..")
+
+        else:
+            for i in zip(["IGO_sub_date", "sequencing_date"], res[0]):
+                if not isinstance(i[1], datetime.date):
+                    logging.error(
+                        "Wrong date format in important dates!\n\t{}\n\t{}".format(
+                            i, db_connect.cur.statement
+                        )
+                    )
+                    logging.warning(
+                        "Sample '{}' will be excluded from processing!".format(
+                            key.split("_")[0]
+                        )
+                    )
+                    failed_samples.append(str(key.split("_")[0]))
+                    logging.warning("Fix the missing dates and try again..")
+                else:
+                    success_list = []
+                    for cmd in sample_set[key]["cmd"]:
+                        dest_bucket_name, dest_object_name, src_data = cmd
+                        msg, success = put_object(
+                            dest_bucket_name,
+                            dest_object_name,
+                            src_data,
+                            kwargs["md5sums"],
+                        )
+                        if success:
+                            success_list.append(success)
+                        else:
+                            failed_samples.append(str(key))
+                    if all(success_list):
+                        try:
+                            db_connect.cur.execute(sample_set[key]["statement"])
+                            print(db_connect.cur.statement)
+                        except mysql.connector.Error as err:
+                            print(
+                                "Something went wrong with sample {}: {}".format(
+                                    key.split("_")[0], err
+                                )
+                            )
+                            db_connect.db.rollback()
+                        finally:
+                            db_connect.db.commit()
+    return failed_samples
+
+
 class JobsYmlKeys:
     def __init__(self, email: str = None, n: int = 1, seqcargs=None, **kargs):
 
@@ -855,6 +977,7 @@ class JobsYmlKeys:
             "Sharp.umiEndPos",
             "Sharp.trimPos",
             "Sharp.slidingWindowSearch",
+            "Sharp.translate10XBarcodes",
             "Sharp.cbCollapsingDistance",
             "Sharp.umiCollapsingDistance",
             "Sharp.numExpectedCells",
@@ -915,9 +1038,9 @@ class JobsYmlKeys:
 
         path = os.path.join(claymldict.pop("s3_loc"), claymldict["sample"], "")
         _, bucket, key, _, _ = urllib.parse.urlsplit(path)
-        R1 = get_objects(bucket, key.strip("/"), re.compile("_R1_\d\d\d.fastq.gz$"))
+        R1 = get_objects(bucket, key.strip("/"), re.compile("_R1_\d{3}.fastq.gz$"))
         R1 = [os.path.join("s3://", bucket, i) for i in R1]
-        R2 = get_objects(bucket, key.strip("/"), re.compile("_R2_\d\d\d.fastq.gz$"))
+        R2 = get_objects(bucket, key.strip("/"), re.compile("_R2_\d{3}.fastq.gz$"))
         R2 = [os.path.join("s3://", bucket, i) for i in R2]
 
         try:
@@ -940,10 +1063,17 @@ class JobsYmlKeys:
         )
         db_connect.cur.execute(stmt)
         res = db_connect.cur.fetchall()
-        hres = all([all(i[:2]) for i in res])
-        if not hres:
+        if not res:
             logging.warning("Barcodes data Empty:\n\t %s", db_connect.cur.statement)
             return str(claymldict["id"])
+        for items in res:
+            try:
+                assert items[0], "AssertionError: Missing sequence barcodes!"
+                assert items[1], "AssertionError: Missing barcode IDs"
+            except AssertionError as err:
+                logging.warning("%s:\n\t %s", err, db_connect.cur.statement)
+                return str(claymldict["id"])
+
         hres = pd.DataFrame(res, columns=["sequence", "code", "label", "bp_shift"])
         seq_length = list(set(map(len, hres.sequence)))
         if len(seq_length) > 1:
@@ -971,14 +1101,22 @@ class JobsYmlKeys:
 
         claymldict["project"] = "Project {}".format(claymldict["project_id"])
         claymldict["sample"] = "_".join([str(claymldict["id"]), claymldict["sample"]])
+        # TODO: make special entry for "CiteSeq": "CiteSeq_results"
         claymldict["destination"] = os.path.join(path, "Hashtag_results", "")
         claymldict["Sharp.scRnaSeqPlatform"] = {
             "H2": "10x_v2",
             "H3": "10x_v3",
             "H4": "in_drop_v4",
+            "CiteSeq": "10x_v3",  # TODO: set CiteSeq platform
         }[claymldict["run_name"]]
         claymldict["Sharp.slidingWindowSearch"] = False
 
+        totalseq = hres.code.str.extract("(^[BC])")
+        if totalseq.dropna().empty:
+            claymldict["Sharp.translate10XBarcodes"] = False
+        else:
+            claymldict["Sharp.translate10XBarcodes"] = True
+
         claymldict["Sharp.cbStartPos"] = 1
         claymldict["Sharp.cbCollapsingDistance"] = 1
         claymldict["Sharp.umiCollapsingDistance"] = 1
@@ -1017,6 +1155,81 @@ class JobsYmlKeys:
 
         return sets
 
+    @classmethod
+    def vdj(cls, **kargs):
+
+        from SCRIdb.upload_stats import get_objects
+        import regex
+
+        vdj_inputs = [
+            "CellRangerVdj.sampleName",
+            "CellRangerVdj.fastqNames",
+            "CellRangerVdj.referenceGenome",
+            "CellRangerVdj.inputFastq",
+        ]
+        vdj_labels = [
+            "pipelineType",
+            "project",
+            "sample",
+            "owner",
+            "destination",
+            "transfer",
+            "comment",
+        ]
+
+        Classicyml = namedtuple("Classicyml", list(kargs.keys()))
+        clayml = Classicyml(**kargs)
+        claymldict = ChainMap(clayml._asdict())
+
+        path = os.path.join(claymldict.pop("s3_loc"), claymldict["sample"], "")
+        _, bucket, key, _, _ = urllib.parse.urlsplit(path)
+        I1 = get_objects(bucket, key.strip("/"), re.compile("_I1_\d{3}.fastq.gz$"))
+        R1 = get_objects(bucket, key.strip("/"), re.compile("_R1_\d{3}.fastq.gz$"))
+        R2 = get_objects(bucket, key.strip("/"), re.compile("_R2_\d{3}.fastq.gz$"))
+
+        try:
+            assert I1, "AssertionError: Missing `I1` index archives!"
+            assert R1, "AssertionError: Missing `R1` barcode archives!"
+            assert R2, "AssertionError: Missing `R2` genomic archives!"
+            assert len(I1) == len(R1) == len(R2), (
+                "AssertionError: number of fastq archives is different!\n\tI1 = "
+                "{}\n\tR1 = {}\n\tR2 = {}".format(len(I1), len(R1), len(R2))
+            )
+            inputFastq = []
+            [inputFastq.extend([l, m, n]) for l, m, n in zip(I1, R1, R2)]
+            inputFastq = [os.path.join("s3://", bucket, str(i)) for i in inputFastq]
+
+        except AssertionError as err:
+            logging.warning("%s\n\t %s", err, path)
+            return str(claymldict["id"])
+
+        claymldict["pipelineType"] = "CellRangerVdj"
+        claymldict["owner"] = os.getlogin()
+        claymldict["transfer"] = "-"
+        claymldict["comment"] = "Cell Ranger V(D)J"
+
+        claymldict["project"] = "Project {}".format(claymldict["project_id"])
+        claymldict["sample"] = "_".join([str(claymldict["id"]), claymldict["sample"]])
+        claymldict["destination"] = os.path.join(path, "CR-vdj-results", "")
+
+        IGO = regex.search(r"IGO(.+?)(?=_\p{L})", R1[0])
+        IGO = IGO.group() if IGO else ""
+        claymldict["CellRangerVdj.sampleName"] = claymldict["sample"]
+        claymldict["CellRangerVdj.fastqNames"] = claymldict["sample"] + IGO
+        claymldict["CellRangerVdj.referenceGenome"] = claymldict.pop("genome_index")
+        claymldict["CellRangerVdj.inputFastq"] = inputFastq
+
+        inputs = os.path.join("config", "{}.inputs.json".format(claymldict["sample"]))
+        labels = os.path.join("config", "{}.labels.json".format(claymldict["sample"]))
+
+        sets = ChainMap(
+            {i: claymldict[i] for i in vdj_inputs},
+            {i: claymldict[i] for i in vdj_labels},
+            {"inputs": inputs, "labels": labels,},
+        )
+
+        return sets
+
     @property
     def appropriate_case(self) -> tuple:
         return self.__appropriatecase
@@ -1024,15 +1237,19 @@ class JobsYmlKeys:
     @appropriate_case.setter
     def appropriate_case(self, mode):
         if mode.lower() not in [
-            i.lower() for i in ["ATAC", "H3", "H2", "H4", "VDJ", "five_prime", "CR"]
+            i.lower()
+            for i in ["ATAC", "H3", "H2", "H4", "VDJ", "five_prime", "CR", "CiteSeq"]
         ]:
             self.__appropriatecase = self.classic(
                 email=self.email, n=self.n, seqcargs=self.seqcargs, **self.kargs,
             )
         elif mode.lower() in [i.lower() for i in ["ATAC", "five_prime", "CR"]]:
             self.__appropriatecase = self.CR(mode=mode.lower(), n=self.n, **self.kargs)
-        elif mode.lower() in [i.lower() for i in ["H3", "H2", "H4"]]:
+        # TODO: CiteSeq appropriate case....
+        elif mode.lower() in [i.lower() for i in ["H3", "H2", "H4", "CiteSeq"]]:
             self.__appropriatecase = self.hashtags(**self.kargs)
+        elif mode.lower() == "VDJ".lower():
+            self.__appropriatecase = self.vdj(**self.kargs)
         else:
             self.__appropriatecase = mode
 
@@ -1040,7 +1257,7 @@ class JobsYmlKeys:
 def jobs_yml_config(
     sample_data: pd.DataFrame,
     config_jobs_yml: str = None,
-    ami: str = "ami-0a4d2955fe21dee72",
+    ami: str = "",
     instance_type: str = "r5.2xlarge",
     star_args: str = "runRNGseed=0",
     email: str = None,
@@ -1066,7 +1283,7 @@ def jobs_yml_config(
         Optional only if running locally.
     :param seqcargs:
         Additional arguments passed to seqc.
-    :param save
+    :param save:
         Save a copy on jobs file to path defined in config_jobs_yml, else return a copy.
 
     :return:
@@ -1160,7 +1377,7 @@ def jobs_yml_config(
         return jobs
 
 
-def json_hasttags(sample_data: pd.DataFrame, config_path, save: bool = True) -> list:
+def json_jobs(sample_data: pd.DataFrame, config_path, save: bool = True) -> list:
     """\
     Constructor for `json` formatted files for batch processing of hashtag samples.
     It Compiles `input` and `label` files for each sample in the provided data frame.
@@ -1170,7 +1387,7 @@ def json_hasttags(sample_data: pd.DataFrame, config_path, save: bool = True) ->
     :param config_path:
         Path to root directory of hashtag pipeline executable `submit.sh`. It also
         serves as the parent directory for `config` where `.json` files will be written.
-    :param save
+    :param save:
         Save a copy on jobs file to path defined in config_jobs_yml, else return a copy.
 
     :return:
@@ -1200,25 +1417,25 @@ def json_hasttags(sample_data: pd.DataFrame, config_path, save: bool = True) ->
             }
         )
     >>> sample_data = sample_data_frame(sd)
-    >>> inputs_labels, exclude_s = json_hasttags(
+    >>> inputs_labels, exclude_s = json_jobs(
             sample_data,
             config_path=os.path.expanduser("~/sharp-0.0.1"),
         )
     """
-    j_hash_, j_labels_ = [], []
+    j_inputs_, j_labels_ = [], []
     inputs_, labels_, exclude_ = [], [], []
     for sample in sample_data.itertuples():
         jobs_json = JobsYmlKeys(**sample._asdict())
         if isinstance(jobs_json.appropriate_case, str):
             exclude_.append(jobs_json.appropriate_case)
         elif jobs_json.appropriate_case:
-            j_hash = jobs_json.appropriate_case.maps[0]
+            j_inputs = jobs_json.appropriate_case.maps[0]
             j_labels = jobs_json.appropriate_case.maps[1]
             inputs = jobs_json.appropriate_case["inputs"]
             labels = jobs_json.appropriate_case["labels"]
             if save:
                 with open(os.path.join(config_path, inputs), "w") as f:
-                    f.write(json.dumps(j_hash, indent=4))
+                    f.write(json.dumps(j_inputs, indent=4))
                     f.close()
                 with open(os.path.join(config_path, labels), "w") as f:
                     f.write(json.dumps(j_labels, indent=4))
@@ -1228,16 +1445,17 @@ def json_hasttags(sample_data: pd.DataFrame, config_path, save: bool = True) ->
                         os.path.join(config_path, "config")
                     )
                 )
-                # write tag-list to AWS
-                hres = jobs_json.appropriate_case["hres"]
-                tag_list = jobs_json.appropriate_case["tag_list"]
-                f = hres.to_csv(header=False, index=False)
-                s3 = boto3.resource("s3")
-                _, bucket, key, _, _ = urllib.parse.urlsplit(tag_list)
-                obj = s3.Object(bucket, key.strip("/"))
-                obj.put(Body=f.encode())
+                # for hashtags write tag-list to AWS
+                if "hres" in jobs_json.appropriate_case:
+                    hres = jobs_json.appropriate_case["hres"]
+                    tag_list = jobs_json.appropriate_case["tag_list"]
+                    f = hres.to_csv(header=False, index=False)
+                    s3 = boto3.resource("s3")
+                    _, bucket, key, _, _ = urllib.parse.urlsplit(tag_list)
+                    obj = s3.Object(bucket, key.strip("/"))
+                    obj.put(Body=f.encode())
             else:
-                j_hash_.append(j_hash)
+                j_inputs_.append(j_inputs)
                 j_labels_.append(j_labels)
 
             inputs_.append(inputs)
@@ -1246,4 +1464,4 @@ def json_hasttags(sample_data: pd.DataFrame, config_path, save: bool = True) ->
     if save:
         return [zip(inputs_, labels_), exclude_]
     else:
-        return [zip(inputs_, labels_, j_hash_, j_labels_), exclude_]
+        return [zip(inputs_, labels_, j_inputs_, j_labels_), exclude_]
