@@ -7,9 +7,9 @@ destination, and grant access to collaborators to their data.
 """
 import re
 import os
+import random
 import string
 import urllib.parse
-from random import choices, shuffle
 from typing import Tuple
 
 import pandas as pd
@@ -44,22 +44,16 @@ def make_credentials(
         `None`
     """
     link = "https://583643567512.signin.aws.amazon.com/console"
-    password = (
-        choices(string.ascii_lowercase)
-        + choices(string.ascii_uppercase)
-        + choices(string.digits)
-        + choices("!@#$%^&*()_+-=[]{}|'")
-        + choices(string.ascii_letters + string.digits + "!@#$%^&*()_+-=[]{}|'", k=4)
+    password = "".join(
+        random.choice(string.ascii_letters + string.digits) for i in range(8)
     )
-    shuffle(password)
-    password = "".join(password)
     iam_ = boto3.client("iam")
     try:
         iam_.create_user(
             UserName=user,
             Tags=[{"Key": "Name", "Value": name}, {"Key": "email", "Value": email}],
         )
-    except ClientError as err:
+    except ClientError:
 
         user = regex.findall("[\p{L}\p{Z}\p{N}_.:/=+\-@]*", user)
         user = "".join(user)
@@ -281,12 +275,33 @@ def data_transfer(
         "H2": "Hashtag_results",
         "H3": "Hashtag_results",
         "H4": "Hashtag_results",
+        # TODO: "CiteSeq": "CiteSeq_results"
+        "CiteSeq": "Hashtag_results",
         "VDJ": "CR-vdj-results",
     }
-
     # read in results_output.csv file with important information
     try:
-        df = pd.read_csv(results_output)
+        df = pd.read_csv(results_output, index_col=0)
+        # set the target destination directory
+        if all(df.destination.isna()):
+            # Default to PI user name
+            if target is None:
+                df["destination"] = df.PI_email.str.split("@", expand=True)[0]
+                df["destination"] = (
+                    "s3://dp-lab-data/collaborators"
+                    + df["destination"].astype(str)
+                    + "/"
+                )
+            else:
+                # otherwise use provided `target`
+                df["destination"] = target
+
+        elif any(df.destination.isna()):
+            print("Fill in the missing `destination` in the table, and try again..")
+            import sys
+
+            sys.exit()
+
     except FileNotFoundError as e:
         try:
             assert results_output == "-", str(e)
@@ -378,16 +393,19 @@ def data_transfer(
                 sample_name=row.Sample, AWS_storage=row.AWS_storage, new_dest=new_dest,
             )
         )
-    # outs = [get_glob(i) for i in df.itertuples()]
     cmds = [i[0] for i in outs]
     dest = [i[1] for i in outs]
     df["command"] = cmds
     df["dest"] = dest
 
     for i in df.itertuples():
+        # TODO: update run_records with new path
+        statement = "UPDATE `raw_data` SET `s3uri`='{}' WHERE sampleData_id={}".format(
+            os.path.join(i.dest, ""), i.id
+        )
         try:
             # update location in database
-            assert i.Run_Name not in ["H2", "H3", "H4", "VDJ"]
+            assert i.Run_Name not in ["H2", "H3", "H4", "VDJ", "CiteSeq"]
             statement1 = "UPDATE {} SET AWS_storage='{}' WHERE id={}".format(
                 i.table.split()[0], os.path.join(i.dest, ""), i.id
             )
@@ -439,7 +457,7 @@ def data_transfer(
                         )
                     )
             statement2 = ""
-        statements = "; ".join([statement1, statement2])
+        statements = "; ".join([statement, statement1, statement2])
         for com in i.command:
             print("Executing command: {}".format(com))
             out, err = execute_cmd(com)
